/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.entity;

import com.iohao.mmo.common.kit.RandomKit;
import lombok.AccessLevel;
import lombok.Data;
import lombok.experimental.FieldDefaults;
import org.springframework.data.mongodb.core.mapping.Document;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * 装备基础属性
 * <pre>
 *     装备的基本属性
 * </pre>
 *
 * @author 唐斌
 * @date 2023-07-24
 */
@Data
@Document
@FieldDefaults(level = AccessLevel.PRIVATE)
public class EquipPropertyBasic {

    /** 物理攻击最小值 */
    Integer physicsAttackMin;
    /** 物理攻击最大值 */
    Integer physicsAttackMax;
    /** 法术攻击最小值 */
    Integer magicAttackMin;
    /** 法术攻击最大值 */
    Integer magicAttackMax;
    /** 物理攻击速度 */
    Double physicsAttackSpeed;
    /** 法术攻击速度 */
    Double magicAttackSpeed;

    /** 物理防御 */
    Integer physicsDefense;
    /** 法术防御 */
    Integer magicDefense;

    /** 物理命中 */
    Integer physicsHit;
    /** 法术命中 */
    Integer magicHit;

    /** 物理躲避 */
    Integer physicsDodge;
    /** 法术躲避 */
    Integer magicDodge;

    /** 生命值 */
    Integer hp;
    /** 能量值 */
    Integer mp;

    /** 基础属性增加百分比 */
    Integer basicAllPct;

    public EquipPropertyBasic() {
    }

    public EquipPropertyBasic(Integer physicsAttackMin, Integer physicsAttackMax, Integer magicAttackMin, Integer magicAttackMax, Double physicsAttackSpeed, Double magicAttackSpeed, Integer physicsDefense, Integer magicDefense, Integer physicsHit, Integer magicHit, Integer physicsDodge, Integer magicDodge, Integer hp, Integer mp) {
        this.physicsAttackMin = physicsAttackMin;
        this.physicsAttackMax = physicsAttackMax;
        this.magicAttackMin = magicAttackMin;
        this.magicAttackMax = magicAttackMax;
        this.physicsAttackSpeed = physicsAttackSpeed;
        this.magicAttackSpeed = magicAttackSpeed;
        this.physicsDefense = physicsDefense;
        this.magicDefense = magicDefense;
        this.physicsHit = physicsHit;
        this.magicHit = magicHit;
        this.physicsDodge = physicsDodge;
        this.magicDodge = magicDodge;
        this.hp = hp;
        this.mp = mp;
    }

    public static EquipPropertyBasic randomFixed(EquipPropertyBasic fixedEquipPropertyMin,
                                                 EquipPropertyBasic fixedEquipPropertyMax){

        EquipPropertyBasic equipPropertyBasic = new EquipPropertyBasic();
        if(fixedEquipPropertyMin==null || fixedEquipPropertyMax == null)
        {
            return equipPropertyBasic;
        }
        try{
            //通过getDeclaredFields()方法获取属性下限对象类中的所有属性（含私有）
            Field[] minFields = fixedEquipPropertyMin.getClass().getDeclaredFields();
            //遍历属性
            for (Field minField : minFields) {
                //设置允许通过反射访问私有变量
                minField.setAccessible(true);
                //获取属性下限类中对应字段的值
                Object minValue = minField.get(fixedEquipPropertyMin);
                //如果获取属性下限类中对应的值为空则跳过
                if(minValue==null){
                    continue;
                }
                //通过getDeclaredFields()方法获取属性上限对象类中的所有属性（含私有）
                Field[] maxFields = fixedEquipPropertyMax.getClass().getDeclaredFields();
                //遍历属性
                for (Field maxField : maxFields) {
                    //设置允许通过反射访问私有变量
                    maxField.setAccessible(true);
                    //获取属性下限类中当前字段属性名称
                    String minName = minField.getName();

                    //获取属性上限类中对应字段的值
                    Object maxValue = maxField.get(fixedEquipPropertyMax);
                    //获取属性上限类中当前字段属性名称
                    String maxName = maxField.getName();
                    //如果下限和上限都取到了同一个属性，并且对应的值都不为空，那么取出来在其中进行范围随机
                    if(minName.equals(maxName)&&maxValue!=null){
                        //在下限和上限之间随机
                        Integer randomValue = RandomKit.randomFromInt(Integer.parseInt(minValue.toString()),
                                Integer.parseInt(maxValue.toString()));
                        //通过属性名称获取对应set方法的名称
                        String methodName = "set" + capitalize(minName);
                        //获取class对象
                        Class<?> clazz = equipPropertyBasic.getClass();
                        //获取赋值的字段类型
                        Class<?> parameterType = randomValue.getClass();
                        //获取set方法
                        Method method = clazz.getMethod(methodName, parameterType);
                        //对equipPropertyBasic的set方法赋值randomValue
                        method.invoke(equipPropertyBasic, randomValue);
                        break;
                    }
                }

            }
        } catch (Exception ex){
            //处理异常
        }
        return equipPropertyBasic;
    }
    private static String capitalize(String str) {
        if (str == null || str.length() == 0) {
            return str;
        }
        char firstChar = str.charAt(0);
        if (Character.isUpperCase(firstChar)) {
            return str;
        } else {
            return Character.toUpperCase(firstChar) + str.substring(1);
        }
    }
    public static void main(String[] args) {
        EquipPropertyBasic fixedEquipPropertyMin = new EquipPropertyBasic();
        fixedEquipPropertyMin.setHp(10);
        fixedEquipPropertyMin.setMp(100);
        EquipPropertyBasic fixedEquipPropertyMax = new EquipPropertyBasic();
        fixedEquipPropertyMax.setHp(50);
        fixedEquipPropertyMax.setMp(200);

        EquipPropertyBasic equipPropertyBasic = randomFixed(fixedEquipPropertyMin,fixedEquipPropertyMax);
        System.out.println(equipPropertyBasic.getHp());
    }
}
