/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.service;

import com.iohao.mmo.common.config.GameCode;
import com.iohao.mmo.equip.entity.EquipPropertyElse;
import com.iohao.mmo.equip.entity.Equip;
import com.iohao.mmo.equip.kit.EquipRandomKit;
import lombok.AllArgsConstructor;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;
import java.math.BigDecimal;
import java.util.List;

/**
 * 装备
 *
 * @author 唐斌
 * @date 2023-07-30
 * @description: 装备属性实现类
 */
@Service
@AllArgsConstructor
public class EquipService {
    final MongoTemplate mongoTemplate;

    /**
     * 通过装备id查询装备
     * @param id 装备id
     * @return
     */
    public Equip findById(String id) {
        return  mongoTemplate.findById(id, Equip.class);
    }

    /**
     * 根据用户查找所有装备
     * @param userId 用户id
     * @return
     */
    public List<Equip> listByUser(long userId) {
        Query query = new Query();
        query.addCriteria(Criteria.where("userId").is(userId));

        return  mongoTemplate.find(query,Equip.class);
    }

    /**
     * 根据装备id列表批量删除装备
     * @param idList 装备id列表
     */
    public void delBatch(List<String> idList) {
        Query query = new Query(new Criteria("id").in(idList));
        mongoTemplate.remove(query, Equip.class);
    }



    /**
     * 重新随机总属性点（鉴定装备）
     * @param id 装备id
     * @param excellentRate 增加的极品率
     */
    public Equip resetEquip(String id, BigDecimal excellentRate){
//        Equip equip = findById(id);
//        GameCode.objNotFound.assertTrue(equip!=null);
//
//        int randomMin = equip.getTotalAttrMin();
//        int randomMax = equip.getTotalAttrMax();
//        //本次是否生成极品
//        boolean excellentFlag = EquipRandomKit.isExcellent(excellentRate);
//        //品质
//        equip.setQuality(excellentFlag?2:1);
//        //额外属性值
//        int newAttrTotal = EquipRandomKit.randomFromExcellent(randomMin,randomMax,excellentFlag);
//        // 直接替换掉原装备的属性值
//        return replaceEquipAttr(equip,newAttrTotal);
        return null;
    }


}
