package com.iohao.mmo.excel.service.impl;

import com.iohao.mmo.excel.kit.GenericsKit;
import com.iohao.mmo.excel.service.BaseService;
import lombok.AllArgsConstructor;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.io.Serializable;
import java.util.*;

/**
 * 基础服务类
 *
 * @author Toby javatangbin@163.com
 */
@Service
@AllArgsConstructor
public abstract class BaseServiceImpl<T> implements BaseService<T> {
    final MongoTemplate mongoTemplate;
    @Override
    public Class<T> currentModelClass() {
        return (Class<T>) GenericsKit.getSuperClassGenricType(getClass(), 0);
    }
    @Override
    public List<T> list(Map<String,Object> param,Class<T> entityClass) {
        Query query = new Query();
        return mongoTemplate.find(query, entityClass);
    }

    @Override
    public T selectById(Serializable id,Class<T> entityClass) {
        return mongoTemplate.findById(id, entityClass);
    }

    @Override
    public void insert(T entity) {
        mongoTemplate.insert(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertBatch(Collection<T> entityList) {
        mongoTemplate.insertAll(entityList);
    }

    @Override
    public void update(T entity) {
        mongoTemplate.save(entity);
    }

    @Override
    public void deleteBatch(List<Serializable> idList,Class<T> entityClass) {
        Query query = new Query(new Criteria("id").in(idList));
        mongoTemplate.remove(query, entityClass);
    }

}