package com.iohao.mmo.excel.service.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.iohao.mmo.excel.entity.DemoExcel;
import com.iohao.mmo.excel.service.DemoExcelService;
import com.mongodb.bulk.BulkWriteResult;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.data.mongodb.core.BulkOperations;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.data.util.Pair;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.util.ArrayList;
import java.util.List;

/**
 * Excel示例
 *
 * @author 唐斌
 * @date 2023-08-30
 * @description: Excel示例
 */
@Service
public class DemoExcelServiceImpl extends BaseServiceImpl<DemoExcel> implements DemoExcelService {
    final MongoTemplate mongoTemplate;

    public DemoExcelServiceImpl(MongoTemplate mongoTemplate) {
        super(mongoTemplate);
        this.mongoTemplate = mongoTemplate;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBatch(List<DemoExcel> entityList) throws JsonProcessingException {
        if (CollectionUtils.isEmpty(entityList)) {
            return;
        }
        List<Pair<Query, Update>> updateList = new ArrayList<>(entityList.size());
        BulkOperations operations = mongoTemplate.bulkOps(BulkOperations.BulkMode.UNORDERED, "demoExcel");
        entityList.forEach(data -> {
            //以code为标识判断新增或者修改
            Query query = new Query(new Criteria("code").is(data.getCode()));
            Update update = new Update();
            //如果userId是主键，必须使用setOnInsert()
            update.set("name",data.getName());
            update.set("remark",data.getRemark());
            update.set("status",data.getStatus());
            update.set("registerDate",data.getRegisterDate());

            Pair<Query, Update> updatePair = Pair.of(query, update);
            updateList.add(updatePair);
        });
        operations.upsert(updateList);
        BulkWriteResult result = operations.execute();
        System.out.println(result.getUpserts());
    }
}
