package com.iohao.mmo.excel.service.impl;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.iohao.mmo.equip.entity.EquipEntry;
import com.iohao.mmo.equip.entity.EquipEntryTemplate;
import com.iohao.mmo.excel.service.EquipEntryTemplateExcelService;
import com.mongodb.bulk.BulkWriteResult;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.data.mongodb.core.BulkOperations;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.data.util.Pair;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.util.ArrayList;
import java.util.List;

/**
 * 装备词条Excel
 *
 * @author 唐斌
 * @date 2023-08-30
 * @description: 装备词条Excel
 */
@Service
public class EquipEntryTemplateExcelServiceImpl extends BaseServiceImpl<EquipEntryTemplate> implements EquipEntryTemplateExcelService {
    final MongoTemplate mongoTemplate;

    public EquipEntryTemplateExcelServiceImpl(MongoTemplate mongoTemplate) {
        super(mongoTemplate);
        this.mongoTemplate = mongoTemplate;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBatch(List<EquipEntryTemplate> entityList) throws JsonProcessingException {
        if (CollectionUtils.isEmpty(entityList)) {
            return;
        }
        List<Pair<Query, Update>> updateList = new ArrayList<>(entityList.size());
        BulkOperations operations = mongoTemplate.bulkOps(BulkOperations.BulkMode.UNORDERED, "equipEntryTemplate");
        entityList.forEach(data -> {
            //以code为标识判断新增或者修改
            Query query = new Query(new Criteria("code").is(data.getCode()));
            Update update = new Update();
            //如果userId是主键，必须使用setOnInsert()
            update.set("name",data.getName());
            update.set("description",data.getDescription());
            update.set("prizeWeight",data.getPrizeWeight());
            update.set("positionList",data.getPositionList());
            update.set("prizeAmount",data.getPrizeAmount());
            update.set("entryPropertyList",data.getEntryPropertyList());

            Pair<Query, Update> updatePair = Pair.of(query, update);
            updateList.add(updatePair);
        });
        operations.upsert(updateList);
        BulkWriteResult result = operations.execute();
        System.out.println(result.getUpserts());
    }
}
