package com.iohao.mmo.excel.service.impl;

import com.iohao.mmo.equip.entity.EquipEntry;
import com.iohao.mmo.equip.entity.EquipTemplate;
import com.iohao.mmo.equip.service.EquipEntryTemplateService;
import com.iohao.mmo.excel.service.EquipTemplateExcelService;
import com.mongodb.bulk.BulkWriteResult;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.BulkOperations;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.data.util.Pair;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.util.ArrayList;
import java.util.List;

/**
 * 装备词条Excel
 *
 * @author 唐斌
 * @date 2023-08-30
 * @description: 装备词条Excel
 */
@Service
public class EquipTemplateExcelServiceImpl extends BaseServiceImpl<EquipTemplate> implements EquipTemplateExcelService {
    final MongoTemplate mongoTemplate;

    @Autowired
    EquipEntryTemplateService equipEntryTemplateService;

    public EquipTemplateExcelServiceImpl(MongoTemplate mongoTemplate) {
        super(mongoTemplate);
        this.mongoTemplate = mongoTemplate;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBatch(List<EquipTemplate> entityList) {
        if (CollectionUtils.isEmpty(entityList)) {
            return;
        }
        List<Pair<Query, Update>> updateList = new ArrayList<>(entityList.size());
        BulkOperations operations = mongoTemplate.bulkOps(BulkOperations.BulkMode.UNORDERED, "equipTemplate");
        entityList.forEach(data -> {
            //以code为标识判断新增或者修改
            Query query = new Query(new Criteria("itemTypeId").is(data.getItemTypeId()));
            Update update = new Update();
            //如果userId是主键，必须使用setOnInsert()
            update.set("position",data.getPosition());
            update.set("level",data.getLevel());
            update.set("quality",data.getQuality());
            update.set("durability",data.getDurability());
            update.set("prizeWeight",data.getPrizeWeight());
            update.set("initCount",data.getInitCount());
            update.set("residualCount",data.getResidualCount());
            update.set("equipPropertyBasicMin",data.getEquipPropertyBasicMin());
            update.set("equipPropertyBasicMax",data.getEquipPropertyBasicMax());
            update.set("equipPropertyFixedMin",data.getEquipPropertyFixedMin());
            update.set("equipPropertyFixedMax",data.getEquipPropertyFixedMax());
            update.set("equipPropertyFixedList",data.getEquipPropertyFixedList());
            update.set("equipPropertyRandomMin",data.getEquipPropertyRandomMin());
            update.set("equipPropertyRandomMax",data.getEquipPropertyRandomMax());

            Pair<Query, Update> updatePair = Pair.of(query, update);
            updateList.add(updatePair);
        });
        operations.upsert(updateList);
        BulkWriteResult result = operations.execute();
        System.out.println(result.getUpserts());
    }

    /**
     * 根据装备等级、编号查找词条模板并随机出词条
     * @param level 等级
     * @param code 编号
     * @return
     */
    @Override
    public EquipEntry entryByCode(int level, String code){
        return equipEntryTemplateService.entryByCode(level,code);
    }

    /**
     * 根据装备等级、编号查找词条模板并随机出词条
     * @param level 等级
     * @param position 部位
     * @param count 随机数量
     * @return
     */
    @Override
    public List<EquipEntry> randomEntryBatch(int level,String position,int count){
        return equipEntryTemplateService.randomEntryBatch(level,position,count);
    }
}
